(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('intersection-observer'), require('react')) :
  typeof define === 'function' && define.amd ? define(['intersection-observer', 'react'], factory) :
  (global = global || self, global.useElementVisibility = factory(null, global.React));
}(this, function (intersectionObserver, react) { 'use strict';

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
  }

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }

  var defaultOptions = {
    multiple: false
  };

  function getDefaultEntry(multiple) {
    var defaultEntry = {
      isIntersecting: false,
      intersectionRatio: 0
    };

    if (multiple) {
      return [];
    }

    return defaultEntry;
  }

  function useElementVisibility() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultOptions;
    var ref = react.useRef(null);
    var observingNodes = react.useRef([]);
    var multiple = options.multiple;

    var _useState = react.useState(getDefaultEntry(multiple)),
        _useState2 = _slicedToArray(_useState, 2),
        visibility = _useState2[0],
        setVisibility = _useState2[1];

    function getObserver() {
      if (ref.current === null) {
        ref.current = new window.IntersectionObserver(function (entries) {
          if (multiple) {
            setVisibility(entries);
          } else {
            setVisibility(entries[0]);
          }
        }, options);
      }

      return ref.current;
    }

    var observer = getObserver();
    var subscribe = react.useCallback(function (node) {
      if (node) {
        observer.observe(node);
        var isDuplicate = observingNodes.current.some(function (currentNode) {
          return currentNode.id === node.id;
        });

        if (!isDuplicate) {
          observingNodes.current.push(node);
        }
      }
    }, [observer]);

    var unsubscribe = function unsubscribe(nodeId) {
      if (!nodeId) {
        observer.disconnect();
        return;
      }

      var nodeToBeUnsubscribed = observingNodes.current.find(function (node) {
        return node.id === nodeId;
      });

      if (nodeToBeUnsubscribed) {
        observer.unobserve(nodeToBeUnsubscribed);
        return;
      }

      throw new Error("Cannot find the node to unsubscribe");
    };

    react.useEffect(function () {
      return function () {
        observer.disconnect();
      };
    }, [observer]);
    return [visibility, subscribe, unsubscribe];
  }

  return useElementVisibility;

}));
